function [negl,p0_est,Q_est,exitflag,optoutput,grad,hessian,timetaken]=get_homogeneous_like(n0,ntrans_cat,tset,gethessian,initrates)
%estimate parameters for a homogeneous CTMC model given initial counts of each state
%and counts of transitions
%SYNTAX
%[negl,p0_est,Q_est,exitflag,optoutput,grad,hessian,timetaken]=get_homogeneous_like(n0,ntrans_cat,tset,gethessian,initrates);
%INPUTS
%n0 is initial numbers in each state
%ntrans_cat is concatenated counts of transitions observed: s rows (where s
%is number of states), s*nt cols (where nt is number of time intervals):
%e.g. [ntrans(1) ntrans(2) ...] from LVlike_sim.m
%tset is the set of time intervals at which observations were taken (first
%observation assumed to be at zero)
%gethessian (optional) is logical: if true, compute the hessian at the
%solution (default: don't compute)
%initrates(optional) is an initial set of rate estimates (the off-diagonal elements of the Q matrix, moving down then across: otherwise estimate an
%average over all the time intervals
%OUTPUTS
%negl is negative log likelihood
%p0_est is estimated initial frequencies
%Q_est is estimated instantaneous rate matrix
%exitflag indicates exit condition of optimizer fmincon
%optoutput is a structure with information about the optimization
%grad is the gradient of the likelihood function at the solution
%hessian is the value of the Hessian (second derivative) matrix at the
%solution
%timetaken is clock time used
%USES
%optimization toolbox (unconstrained optimization: fminunc)
%QvecfromP.m
%Qfromest.m
%CTMClikefun.m (likelihood function)
%homlike_objfun is nested function that calls CTMClikefun

if nargin<4
    gethessian=false;
end;
if nargin<5
    initrates=[];
end;

tzero=clock;
ncat=length(n0);

%set options
options=optimset('fminunc');
nvar=(ncat+1)*(ncat-1);%number of parameters in this model
options=optimset(options,'largescale','off','display','iter','MaxFunEvals',1000*nvar,'MaxIter',1000);

%set initial estimates
%make the initial frequencies equal to the multinomial estimates
if size(n0,2)>1
    n0=n0'; %we want a column vector
end
p0_est=n0/sum(n0);%this is always the ML estimate for this model

%initial estimate of rate matrix
if isempty(initrates)
    tt=[0 tset];
    Qav=zeros(ncat*(ncat-1),1);
    ind=1:ncat;
    for t=2:length(tt)
        nobs=ntrans_cat(:,ind)+1;%select the right cols of ntrans_cat and use Laplace pseudocounts
        ind=ind+ncat;
        csum=sum(nobs);
        temp=csum(ones(ncat,1),:);
        thisP=nobs./temp;
        Qav=Qav+QvecfromP(thisP,tt(t)-tt(t-1)); %real part of logm(P), negative elements set to tiny +ve
    end;
    Qav=Qav/length(tset);%average of estimates over each time interval
    init=log(Qav);%so we get positive estimates from unconstrained optimization
else%use the specified initial rate matrix
    init=log(initrates);
end;
%call CTMClikefun.m via fminunc and nested function homlike_objfun to optimize parameters
if gethessian
    [est,negl,exitflag,optoutput,grad,hessian]=fminunc(@homlike_objfun,init,options);
else
    [est,negl,exitflag,optoutput,grad]=fminunc(@homlike_objfun,init,options);
    hessian=[];
end;

%reconstruct the outputs we want from est
est=exp(est);%back-transform
Q_est=Qfromest(est,ncat);

timetaken=etime(clock,tzero);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %nested function for optimization: use this so we have access to
    %n0,ntrans_cat and tset
    function f=homlike_objfun(xval)
        est=exp(xval);%back-transform
        Q=Qfromest(est,ncat);%turn into matrix again
        f=-CTMClikefun(tset,n0,ntrans_cat,Q);%get the full negative log likelihood
        
        if ~isfinite(f)%sometimes things like expm fail and we get -Inf
            f=0;%this should mean optimization stops soon after...
        end;
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end

